/* Simple Video4Linux image grabber. Made for my Philips Vesta Pro
 * 
 * Copyright (C) 2000 geci, Larchmont, USA
 * eMail: devel@gecius.de
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at you option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public Licence for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <linux/types.h>
#include <linux/videodev.h>
#include <jpeglib.h>

#define QUAL_DEFAULT 75
#define VIDEO_DEV "/dev/video"
#define OUT_DEFAULT "/dev/stdout"

int get_brightness_adj(unsigned char *image, long size, int *brightness) {
    long i, tot = 0;
    for (i=0;i<size*3;i++)
    tot += image[i];
    *brightness = (128 - tot/(size*3))/3;
    return !((tot/(size*3)) >= 126 && (tot/(size*3)) <= 130);
}

void usage (char *pname) {
    fprintf(stderr,
	"%s, Version %s\n"
	"Usage: %s [options]\n"
	" -h, --help        This screen\n"
	" -l <seconds>      Loop forever with sleep for <seconds>\n"
	" -q <quality>      Quality setting (1-100, default: %d)\n"
	" -f <filename>     Write to <filename> (default: stdout)\n"
	" -n                Disable the dot for each image (for daemon use)\n"
	" -d <device>       Read from <device> as input (default: %s)\n"
	"\n"
	"Example: %s -l 5 -f /usr/local/image.jpg\n"
	"         Would write a single jpeg-image to image.jpg every five seconds\n"
	"\n"
	"Currently, the only output format is jpeg and the video stream has to\n"
	"be rgb24 or grayscale. Sorry folks.\n",
	basename(pname), VERSION, basename(pname), QUAL_DEFAULT, VIDEO_DEV, basename(pname));
	exit (1);
}

int main(int argc, char *argv[]) {
    int y, dev, line_width, f, n, bpp = 24;
    boolean loop=FALSE;
    boolean logstd=TRUE;
    int quality= QUAL_DEFAULT;
    int sleeptime;
    char *in = VIDEO_DEV;
    char *out = OUT_DEFAULT;
    char *line;
    unsigned char *buffer;
    unsigned int i, src_depth;

    FILE *x;
    JSAMPROW row_ptr[1];

    struct video_capability cap;
    struct video_window win;
    struct video_picture vpic;
    struct jpeg_compress_struct cjpeg;
    struct jpeg_error_mgr jerr;

    while ((n = getopt (argc, argv, "l:f:q:hd:n"))!=EOF) {
	switch (n) {
	    case 'l':
		loop = TRUE;
		sscanf (optarg, "%d", &sleeptime);
		break;
	    case 'f':
		out = optarg;
		break;
	    case 'q':
		sscanf (optarg, "%d", &quality);
		break;
	    case 'd':
		in = optarg;
		break;
	    case 'n':
		logstd=FALSE;
		break;
	    default:
		usage (argv[0]);
		break;
	}
    }

    do {
        dev = open(in, O_RDONLY);

        if (dev < 0) {
	    perror(in);
	    exit(1);
	}

        if (ioctl(dev, VIDIOCGCAP, &cap) < 0) {
            perror("VIDIOGCAP");
            close(dev);
            exit(1);
        }

        if (ioctl(dev, VIDIOCGWIN, &win) < 0) {
            perror("VIDIOCGWIN");
            close(dev);
            exit(1);
        }

        if (ioctl(dev, VIDIOCGPICT, &vpic) < 0) {
            perror("VIDIOCGPICT");
            close(dev);
            exit(1);
	}
        vpic.depth=24;
        vpic.palette=VIDEO_PALETTE_RGB24;

	buffer = malloc(win.width * win.height * bpp);
	if (!buffer) {
	    exit(1);
	}
	do {
	    int newbright;
	    read(dev, buffer, win.width * win.height * bpp);
	    f = get_brightness_adj(buffer, win.width * win.height, &newbright);
	    if (f) {
		vpic.brightness += (newbright << 8);
		if(ioctl(dev, VIDIOCSPICT, &vpic)==-1) {
		    perror("VIDIOSPICT");
		    break;
		}
	    }
	} while (f);
	close(dev);
	line=malloc(win.width * 3);
	if (!line) {
	    perror("OUT OF MEMORY");
	    exit(1);
	}
	cjpeg.err = jpeg_std_error(&jerr);
        jpeg_create_compress (&cjpeg);
	cjpeg.image_width = win.width;
        cjpeg.image_height= win.height;
	cjpeg.input_components = 3;
        cjpeg.in_color_space = JCS_RGB;

	jpeg_set_defaults (&cjpeg);
	jpeg_set_quality (&cjpeg, quality, TRUE);

	cjpeg.dct_method = JDCT_FASTEST;

	x = fopen(out, "w+");

	jpeg_stdio_dest (&cjpeg, x);
	jpeg_start_compress (&cjpeg, TRUE);

	row_ptr[0]=line;
	line_width=win.width * 3;
	n=0;
	for (y = 0; y < win.height; y++) {
	    for (i = 0; i< line_width; i+=3) {
		line[i]   = buffer[n+2];
		line[i+1] = buffer[n+1];
		line[i+2] = buffer[n];
		n+=3;
	    }
	    jpeg_write_scanlines (&cjpeg, row_ptr, 1);
	}
	jpeg_finish_compress (&cjpeg);
	fclose(x);
	jpeg_destroy_compress (&cjpeg);
	free (line);
	free (buffer);
	if (loop) {
	    if (logstd) {
		fprintf(stderr,".");
	    }
	    sleep(sleeptime);
	}
    } while (loop);
}
